#ifndef __LIBUISM_H__
#define __LIBUISM_H__

/*-------------------------------------------------------------------------------------------------------*/
#if defined(__linux__) || defined(__QNX__)
	#if (__GNUC__ > 3) && (defined(__ELF__) || defined(__PIC__))
	#    define INNO_API 		__attribute__((__visibility__("default")))
	#    define INNO_PRIVATE 	__attribute__((__visibility__("hidden")))
	#else
	#    define INNO_API
	#    define INNO_PRIVATE
	#endif
#endif

#define DIM(var, type)  (sizeof(var)) / sizeof(type)

/*-------------------------------------------------------------------------------------------------------*/
#define VER_LEN						16
#define PID_LEN						8
#define FW_LEN						8
#define SN_LEN						16

/* Sensor name */
#define SENSOR_NAME_HCHO			0x00000001
#define SENSOR_NAME_PM				0x00000002
#define SENSOR_NAME_CO2				0x00000004
#define SENSOR_NAME_CO				0x00000008
#define SENSOR_NAME_O3				0x00000010
#define SENSOR_NAME_TVOC			0x00000020
#define SENSOR_NAME_TH				0x00000040	/* Temperature & Humidity */

#define SENSOR_NAME_MONITOR_NULL	0x00000000
#define SENSOR_NAME_MONITOR_ALL		0xFFFFFFFF

/* Return code */
#define	UISM_SUCCESS				0	/* success 														*/

#define	UISM_HID_FOUND_ERR			1	/* found HID device error 										*/
#define	UISM_HID_NOT_FOUND			2	/* HID device can't found 										*/
#define	UISM_HID_OPEN_ERR			3	/* open HID device error										*/
#define	UISM_HID_CLOSE_ERR			4	/* close HID device error										*/
#define	UISM_HID_INIT_ERR			5	/* initial HID device error										*/
#define	UISM_HID_EXIT_ERR			6	/* exit HID device error										*/

#define	UISM_I2C_FREQ_ERR			11	/* set I2C frequency error										*/
#define	UISM_I2C_ADDR_ERR			12	/* set I2C address error										*/
#define	UISM_I2C_GET_INFO_ERR		13	/* I2C get information error									*/

#define	UISM_LIB_OPEN_ONLY			21	/* This function is only supported when the device is opened	*/
#define	UISM_LIB_CLOSE_ONLY			22	/* This function is only supported when the device is closed	*/
#define	UISM_LIB_START_ONLY			23	/* This function is only supported when the device is started	*/
#define	UISM_LIB_STOP_ONLY			24	/* This function is only supported when the device is stopped	*/
#define UISM_LIB_INVALID_VAR		25	/* Input invalid variable to API */

#define	UISM_SNR_INVALID			31	/* invalid sensor name											*/

/*-------------------------------------------------------------------------------------------------------*/
/*
 *	SENSOR_NAME		data1 (unit)										data2 (unit)		ststus
 *	HCHO			Formaldehyde		   			 (0.01 ppm)			X					X
 *	PM				PM2.5				   			 (0.1 ug/m^3)		PM10 (0.1 ug/m^3)	X
 *	CO2				CO2					   			 (1 ppm)			X					0: Normal operation / 1: Warm-Up phase / 2: Error case
 *	CO				CO					   			 (0.1 ppm)			X					0: Normal operation / 1: Warm-Up phase / 2: Error case
 *	O3				O3					   			 (0.01 ppm)			X					0: Normal operation / 1: Warm-Up phase / 2: Error case
 *	TVOC			Total volatile organic compounds (0.001 ppm)		X					0: Normal operation / 1: Warm-Up phase / 2: Error case
 *	TH				Temperature & Humidity 			 (0.01 C & 0.01 %)	X					X
 */
typedef struct
{
	unsigned char *data1;
	int			   data1_len;

	unsigned char *data2;
	int			   data2_len;

	unsigned char  status;

} DATA_INFO;

typedef void (*DATA_CB) (int error_code, int name, DATA_INFO *info);

typedef struct
{
	char api[VER_LEN];

} VER_INFO;

typedef struct
{
	/*----- input variable from user to API  -----*/
	int			  name;

	/*----- output variable from API to user -----*/
	unsigned char PID[PID_LEN + 1];
	unsigned char FW_ver[FW_LEN + 1];
	unsigned char SN[SN_LEN + 1];

	unsigned char status;

} SENSOR_INFO;

typedef struct
{
	unsigned int interval;	/* unit: ms */
	int			 monitor_opt;
	bool		 check_status;
	DATA_CB		 data_cb;

} INIT_INFO;

/*-------------------------------------------------------------------------------------------------------*/
/* Debug level */
enum
{
	DBG_LVL_NONE = 0,
	DBG_LVL_FUNC,
	DBG_LVL_HID,
};

/* INIT */
enum
{
	INIT_STS_STOP = 0,
	INIT_STS_START,
};

/*-------------------------------------------------------------------------------------------------------*/
#if defined(__linux__) || defined(__QNX__)
	extern INNO_API int  UISMShowVer	 (VER_INFO *ver_info);
	extern INNO_API int  UISMDebugLevel	 (int level);
	extern INNO_API int  UISMOpenDevice	 (void);
	extern INNO_API int  UISMCloseDevice (void);
	extern INNO_API int	 UISMSensorInfo	 (SENSOR_INFO *sensor_info);
	extern INNO_API int  UISMInit		 (bool status, INIT_INFO *init_info);
	extern INNO_API	int  UISMQuery 	     (int *result);
#else

#if defined(_MSC_VER) /* for Visual Studio */
	typedef int  (*UISM_SHOW_VER) 		(VER_INFO *ver_info);
	typedef int  (*UISM_DEBUG_LEVEL)	(int level);
	typedef int  (*UISM_OPEN_DEVICE)	(void);
	typedef int  (*UISM_CLOSE_DEVICE)	(void);
	typedef int  (*UISM_SENSOR_INFO)	(SENSOR_INFO *sensor_info);
	typedef int  (*UISM_INIT)			(bool status, INIT_INFO *init_info);
	typedef int  (*UISM_Query)			(int *result);
#else
	extern int 	UISMShowVer		(VER_INFO *ver_info);
	extern int 	UISMDebugLevel	(int level);
	extern int 	UISMOpenDevice	(void);
	extern int 	UISMCloseDevice	(void);
	extern int	UISMSensorInfo	(SENSOR_INFO *sensor_info);
	extern int 	UISMInit		(bool status, INIT_INFO *init_info);
	extern int 	UISMQuery 		(int *result);
#endif

#endif

#endif
